# -*- coding: utf-8 -*-
from uamobile import exceptions
from uamobile.base import UserAgent, Display
from uamobile.docomodisplaymap import DISPLAYMAP_DOCOMO

import re

try:
    frozenset
except NameError:
    from sets import Set as frozenset

STATUS_SET = frozenset(['TC', 'TD', 'TB', 'TJ'])

DISPLAY_BYTES_RE = re.compile(r'^W(\d+)H(\d+)$')
VENDOR_RE = re.compile(r'([A-Z]+)\d')

FOMA_SERIES_4DIGITS_RE = re.compile(r'\d{4}')
FOMA_SERIES_3DIGITS_RE = re.compile(r'(\d{3}i)')

class DoCoMoUserAgent(UserAgent):
    """
    NTT DoCoMo implementation
    see also http://www.nttdocomo.co.jp/service/imode/make/content/spec/useragent/index.html

    property "cache" returns cache size as kilobytes unit.

    property "status" means:
    TB | Browsers
    TC | Browsers with image off (only Available in HTTP 5.0)
    TD | Fetching JAR
    TJ | i-Appli
    """
    carrier = 'DoCoMo'
    short_carrier = 'D'

    HTML_VERSION_MAP = (((re.compile('[DFNP]501i'), '1.0'),
                         (re.compile('502i|821i|209i|651|691i|(?:F|N|P|KO)210i|^F671i$'), '2.0'),
                         (re.compile('(?:D210i|SO210i)|503i|211i|SH251i|692i|200[12]|2101V'), '3.0'),
                         (re.compile('504i|251i|^F671iS$|212i|2051|2102V|661i|2701|672i|SO213i|850i'), '4.0'),
                         (re.compile('eggy|P751v'), '3.2'),
                         (re.compile('505i|252i|900i|506i|880i|253i|P213i|901i|700i|851i$|701i|881i|^SA800i$|600i|^L601i$|^M702i(?:S|G)$|^L602i$'), '5.0'),
                         (re.compile('902i|702i|851iWM|882i|883i$|^N601i$|^D800iDS$|^P70[34]imyu$'), '6.0'),
                         (re.compile('883iES|903i|703i|904i|704i'), '7.0'),
                         (re.compile('905i|705i'), '7.1'),
                         (re.compile('906i'), '7.2'),
                         ))

    def __init__(self, *args, **kwds):
        UserAgent.__init__(self, *args, **kwds)
        self.version = ''
        self.bandwidth = None
        self.serialnumber = None
        self.card_id = None
        self.comment = None
        self.cache_size = 5
        self._display_bytes = None
        self._is_foma = False

    def is_docomo(self):
        return True

    def get_html_version(self):
        """
        returns supported HTML version like '3.0'.
        if unkwon, return None.
        """
        for pattern, value in self.HTML_VERSION_MAP:
            if pattern.search(self.model):
                return value
        return None
    html_version = property(get_html_version)

    def get_vendor(self):
        """
        returns vender code like 'SO' for Sony.
        if unkwon, returns None
        """
        matcher = VENDOR_RE.match(self.model)
        if matcher:
            return matcher.group(1)
        else:
            return None
    vendor = property(get_vendor)

    def get_series(self):
        """
        returns series name like '502i'.
        if unknow, return None.
        """
        if self._is_foma and FOMA_SERIES_4DIGITS_RE.search(self.model):
            return 'FOMA'

        matcher = FOMA_SERIES_3DIGITS_RE.search(self.model)
        if matcher:
            return matcher.group(1)

        if self.model == 'P651ps':
            return '651'

        return None
    series = property(get_series)

    def is_gps(self):
        return self.model in ('F661i', 'F505iGPS')

    def is_foma(self):
        return self._is_foma

    def supports_cookie(self):
        return False

    def get_guid(self):
        """
        Get iMode ID(guid). For iMode ID, see
        http://www.nttdocomo.co.jp/service/imode/make/content/ip/index.html#imodeid
        """
        try:
            return self.environ['HTTP_X_DCMGUID']
        except KeyError:
            return None
    guid = property(get_guid)

    def make_display(self):
        """
        create a new Display object.
        """
        try:
            params = DISPLAYMAP_DOCOMO[self.model]
        except KeyError:
            params = {}

        if self._display_bytes:
            try:
                params['width_bytes'], params['height_bytes'] = map(int, self._display_bytes)
            except ValueError:
                pass

        return Display(**params)

    def parse(self):
        main, foma_or_comment = (self.useragent.split(' ', 1) + [None])[:2]
        if not foma_or_comment:
            # DoCoMo/1.0/R692i/c10
            self._parse_main(main)
        elif foma_or_comment[-1] != ')':
            raise exceptions.NoMatchingError(self)
        else:
            if foma_or_comment[0] == '(':
                # DoCoMo/1.0/P209is (Google CHTML Proxy/1.0)
                self.comment = foma_or_comment[1:-1]
                self._parse_main(main)
            else:
                # DoCoMo/2.0 N2001(c10;ser0123456789abcde;icc01234567890123456789)
                self._is_foma = True
                xxx, self.version = main.split('/')
                self._parse_foma(foma_or_comment)

    def _parse_main(self, main):
        """
        parse main part of HTTP_USER_AGENT string (not foma)
        """
        xxx, self.version, self.model, cache, rest = (main.split('/', 4) + [None, None, None, None])[:5]
        if self.model == 'SH505i2':
            self.model = 'SH505i'

        if cache:
            try:
                self.cache_size = int(cache[1:])
            except ValueError:
                raise exceptions.NoMatchingError(self)

        if rest:
            for value in rest.split('/'):
                if value in STATUS_SET:
                    self.status = value
                    continue

                if value.startswith('ser') and len(value) == 14:
                    self.serialnumber = value[3:]
                    continue

                if value.startswith('s'):
                    try:
                        self.bandwidth = int(value[1:])
                        continue
                    except ValueError:
                        pass

                matcher = DISPLAY_BYTES_RE.match(value)
                if matcher:
                    self._display_bytes = matcher.groups()

    def _parse_foma(self, foma):
        try:
            self.model, foma_params = foma.split('(', 1)
        except ValueError:
            raise exceptions.NoMatchingError(self)

        if self.model == 'MST_v_SH2101V':
            self.model = 'SH2101V'

        # for crawlers, such as
        # DoCoMo/2.0 N902iS(c100;TB;W24H12)(compatible; moba-crawler; http://crawler.dena.jp/)
        # DoCoMo/2.0 P900i(c100;TB;W24H11)(compatible; ichiro/mobile goo; +http://help.goo.ne.jp/door/crawler.html)
        # DoCoMo/2.0 SO902i(c100;TB;W20H10) (symphonybot1.froute.jp; +http://search.froute.jp/howto/crawler.html)
        foma_params = foma_params.split(')', 1)[0]

        for value in foma_params.split(';'):
            if value in STATUS_SET:
                self.status = value
                continue

            if value.startswith('ser') and len(value) == 18:
                self.serialnumber = value[3:]
                continue

            if value.startswith('icc') and len(value) == 23:
                self.card_id = value[3:]
                continue

            if value.startswith('c'):
                try:
                    self.cache_size = int(value[1:])
                    continue
                except ValueError:
                    pass

            matcher = DISPLAY_BYTES_RE.match(value)
            if matcher:
                self._display_bytes = matcher.groups()
                continue




# -*- coding: utf-8 -*-
from tests import msg
from uamobile import detect, EZweb

def test_netfront_nonmobile_mode():
    ua = detect({'HTTP_USER_AGENT':'KDDI-TS3A UP.Browser/6.2.0.11.2.1 (GUI) MMP/2.0, Mozilla/4.08 (MobilePhone; NMCS/3.3) NetFront/3.3'})
    assert ua.is_ezweb()
    assert ua.model == 'TS3A'
    assert ua.version == '6.2.0.11.2.1 (GUI)'
    assert ua.server == 'MMP/2.0'

def test_crawler():
    def func(useragent):
        ua = detect({'HTTP_USER_AGENT': useragent})
        assert ua.is_ezweb()

    for useragent in ('KDDI-CA23 UP.Browser/6.2.0.5 (compatible; Y!J-SRD/1.0; http://help.yahoo.co.jp/help/jp/search/indexing/indexing-27.html)',
                      'KDDI-CA31 UP.Browser/6.2.0 (GUI) (compatible; ichiro/mobile goo; +http://help.goo.ne.jp/door/crawler.html)',
                      'KDDI-HI31 UP.Browser/6.2.0.5 (GUI) MMP/2.0 (compatible; LD_mobile_bot; +http://helpguide.livedoor.com/help/search/qa/grp627)',
                      'KDDI-SH32 UP.Browser/6.2.0.6.2 (GUI) MMP/2.0 (symphonybot1.froute.jp; +http://search.froute.jp/howto/crawler.html)',
                      ):
        yield func, useragent

def test_no_subscription_id():
    ua = detect({'HTTP_USER_AGENT': 'KDDI-SA35 UP.Browser/6.2.0.9.1 (GUI) MMP/2.0'})
    assert ua.serialnumber is None

def test_display():
    env = {'HTTP_USER_AGENT': 'KDDI-SA35 UP.Browser/6.2.0.9.1 (GUI) MMP/2.0',
           'HTTP_X_UP_DEVCAP_MAX_PDU': '131072',
           'HTTP_X_UP_DEVCAP_MULTIMEDIA': '9300941123301120',
           'HTTP_X_UP_DEVCAP_SCREENPIXELS': '240,268',
           'HTTP_X_SSL': 'off',
           'HTTP_X_UP_DEVCAP_SOFTKEYSIZE': '6',
           'HTTP_X_UP_DEVCAP_SCREENDEPTH': '16,RGB565',
           'HTTP_X_UP_DEVCAP_NUMSOFTKEYS': '2',
           'HTTP_X_UP_PROXY_BA_REALM': 'realm',
           'HTTP_X_UP_DEVCAP_QVGA': '1',
           'HTTP_X_UP_DEVCAP_SCREENCHARS': '23,12',
           'HTTP_X_UP_DEVCAP_CC': '1',
           'HTTP_X_UP_DEVCAP_TITLEBAR': '1',
           'HTTP_X_UP_DEVCAP_ISCOLOR': '1',
           'HTTP_X_UP_SUBNO': '00000000000000_ef.ezweb.ne.jp' }
    ua = detect(env)
    assert ua.display.width == 240, "%s expected, result %r" % (240, ua.display.width)
    assert ua.display.height == 268, "%s expected, result %r" % (268, ua.display.height)
    assert ua.display.color == True
    assert ua.display.is_vga() == False
    assert ua.display.is_qvga() == True

def test_display_error():
    base_env = {'HTTP_USER_AGENT': 'KDDI-SA35 UP.Browser/6.2.0.9.1 (GUI) MMP/2.0',
                'HTTP_X_UP_DEVCAP_SCREENPIXELS': '240,268',
                'HTTP_X_UP_DEVCAP_SCREENDEPTH': '16,RGB565',
                'HTTP_X_UP_DEVCAP_ISCOLOR': '1' }

    # No ISCOLOR
    env = dict(base_env)
    del env['HTTP_X_UP_DEVCAP_ISCOLOR']
    ua = detect(env)
    assert ua.display.width == 240
    assert ua.display.height == 268
    assert ua.display.depth
    assert ua.display.color == False

    # invalid SCREENPIXELS
    env = dict(base_env)
    del env['HTTP_X_UP_DEVCAP_SCREENPIXELS']
    ua = detect(env)
    assert ua.display.width == 0
    assert ua.display.height == 0
    assert ua.display.color
    assert ua.display.depth

    env = dict(base_env)
    env['HTTP_X_UP_DEVCAP_SCREENPIXELS'] = '240,egg'
    ua = detect(env)
    assert ua.display.width == 0
    assert ua.display.height == 0
    assert ua.display.color
    assert ua.display.depth

    env = dict(base_env)
    env['HTTP_X_UP_DEVCAP_SCREENPIXELS'] = '240,268,1'
    ua = detect(env)
    assert ua.display.width == 0
    assert ua.display.height == 0
    assert ua.display.color
    assert ua.display.depth

    # invalid SCREENDEPTH
    env = dict(base_env)
    del env['HTTP_X_UP_DEVCAP_SCREENDEPTH']
    ua = detect(env)
    assert ua.display.width == 240
    assert ua.display.height == 268
    assert ua.display.color
    assert ua.display.depth == 0

    env = dict(base_env)
    env['HTTP_X_UP_DEVCAP_SCREENDEPTH'] = 'spam'
    ua = detect(env)
    assert ua.display.width == 240
    assert ua.display.height == 268
    assert ua.display.color
    assert ua.display.depth == 0

def test_useragent_ezweb():
    def inner(useragent, version, model, device_id, server, xhtml_compliant, comment, is_wap1, is_wap2):
        ua = detect({'HTTP_USER_AGENT':useragent})
        assert isinstance(ua, EZweb)
        assert ua.carrier == 'EZweb'
        assert ua.short_carrier == 'E'
        assert ua.is_docomo() == False
        assert ua.is_ezweb()
        assert ua.is_softbank() == False
        assert ua.is_vodafone() == False
        assert ua.is_jphone() == False
        assert ua.is_willcom() == False
        assert ua.model == model, msg(ua, ua.model, model)
        assert ua.version == version, msg(ua, ua.version, version)
        assert ua.device_id == device_id, msg(ua, ua.device_id, device_id)
        assert ua.server == server, msg(ua, ua.server, server)
        assert ua.comment == comment, msg(ua, ua.comment, comment)
        assert ua.is_xhtml_compliant() == xhtml_compliant
        assert ua.is_wap1() == is_wap1
        assert ua.is_wap2() == is_wap2
        assert ua.display is not None
        assert ua.supports_cookie() == True

    for args in DATA:
        yield ([inner] + list(args))

#########################
# Test data
#########################

DATA = (
    # ua, version, model, device_id, server, xhtml_compliant, comment, is_wap1, is_wap2
    ('UP.Browser/3.01-HI01 UP.Link/3.4.5.2', '3.01', 'HI01', 'HI01', 'UP.Link/3.4.5.2', False, None, True, False),
    ('KDDI-TS21 UP.Browser/6.0.2.276 (GUI) MMP/1.1', '6.0.2.276 (GUI)', 'TS21', 'TS21', 'MMP/1.1', True, None, False, True),
    ('UP.Browser/3.04-TS14 UP.Link/3.4.4 (Google WAP Proxy/1.0)', '3.04', 'TS14', 'TS14', 'UP.Link/3.4.4', False, 'Google WAP Proxy/1.0', True, False),
    ('UP.Browser/3.04-TST4 UP.Link/3.4.5.6', '3.04', 'TST4', 'TST4', 'UP.Link/3.4.5.6', False, None, True, False),
    ('KDDI-KCU1 UP.Browser/6.2.0.5.1 (GUI) MMP/2.0', '6.2.0.5.1 (GUI)', 'KCU1', 'KCU1', 'MMP/2.0', True, None, False, True),
    ('KDDI-TS2A UP.Browser/6.2.0.9 (GUI) MMP/2.0', '6.2.0.9 (GUI)', 'TS2A', 'TS2A', 'MMP/2.0', True, None, False, True),
)



# -*- coding: utf-8 -*-
from tests import msg
from uamobile import detect, Willcom

def test_useragent():
    def inner(useragent, name, vendor, model, model_version, browser_version, cache_size):
        ua = detect({'HTTP_USER_AGENT': useragent})

        assert ua.is_willcom()
        assert isinstance(ua, Willcom)
        assert ua.name == name
        assert ua.carrier == 'WILLCOM'
        assert ua.short_carrier == 'W'
        assert ua.vendor == vendor
        assert ua.model == model, ua.model
        assert ua.model_version == model_version, ua.model_version
        assert ua.browser_version == browser_version
        if cache_size is not None:
            assert ua.cache_size == cache_size
        else:
            assert ua.cache_size is None
        assert ua.display is not None
        assert ua.serialnumber is None
        assert ua.supports_cookie() == True

    for args in DATA:
        yield ([inner] + list(args))

#########################
# Test data
#########################

# ua, name, vendor, model, model_version, browser_version, cache_size
DATA = (('Mozilla/3.0(DDIPOCKET;JRC/AH-J3001V,AH-J3002V/1.0/0100/c50)CNF/2.0', 'WILLCOM', 'JRC', 'AH-J3001V,AH-J3002V', '1.0', '0100', 50),
        ('Mozilla/3.0(DDIPOCKET;KYOCERA/AH-K3001V/1.7.2.70.000000/0.1/C100) Opera 7.0', 'WILLCOM', 'KYOCERA', 'AH-K3001V', '1.7.2.70.000000', '0.1', 100),
        ('Mozilla/3.0(DDIPOCKET;JRC/AH-J3003S/1.0/0100/c50)CNF/2.0', 'WILLCOM', 'JRC', 'AH-J3003S', '1.0', '0100', 50),
        ('Mozilla/3.0(WILLCOM;SANYO/WX310SA/2;1/1/C128) NetFront/3.3,61.198.142.127', 'WILLCOM', 'SANYO', 'WX310SA', '2;1', '1', 128),

        # IE on W-zero3
        ('Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; SHARP/WS007SH; PPC; 480x640)', 'WILLCOM', 'SHARP', 'WS007SH', '', '', None),

        # Opera on W-zero3
        ('Mozilla/4.0 (compatible; MSIE 6.0; Windows CE; SHARP/WS007SH; PPC; 480x640) Opera 8.60 [ja]', 'WILLCOM', 'SHARP', 'WS007SH', '', '', None),
)



# -*- coding: utf-8 -*-
from uamobile import exceptions
from uamobile.base import UserAgent, Display
import re

VODAFONE_VENDOR_RE = re.compile(r'V\d+([A-Z]+)')
JPHONE_VENDOR_RE = re.compile(r'J-([A-Z]+)')
CRAWLER_RE = re.compile(r'^(.+?)\([^)]+\)$')

class SoftBankUserAgent(UserAgent):
    carrier = 'SoftBank'
    short_carrier = 'S'

    def __init__(self, *args, **kwds):
        UserAgent.__init__(self, *args, **kwds)
        self.packet_compliant = False
        self.serialnumber = None
        self.vendor = ''
        self.vendor_version = None
        self.java_info = {}
        self._is_3g = True
        self.msname = ''

    def supports_cookie(self):
        """
        returns True if the device supports HTTP Cookie.
        For more information, see
        http://www2.developers.softbankmobile.co.jp/dp/tool_dl/download.php?docid=119
        """
        return self.is_3g() or self.is_type_w()

    def is_softbank(self):
        return True

    def is_vodafone(self):
        return True

    def is_jphone(self):
        return True

    def is_3g(self):
        return self._is_3g

    def is_type_c(self):
        """
        returns True if the type is C.
        """
        return not self._is_3g and (self.version.startswith('3.') or self.version.startswith('2.'))

    def is_type_p(self):
        """
        returns True if the type is P.
        """
        return not self._is_3g and self.version.startswith('4.')

    def is_type_w(self):
        """
        returns True if the type is W.
        """
        return not self._is_3g and self.version.startswith('5.')

    def get_jphone_uid(self):
        """
        returns the x-jphone-uid
        for the information about x-jphone-uid, see
        http://developers.softbankmobile.co.jp/dp/tool_dl/web/tech.php
        """
        try:
            return self.environ['HTTP_X_JPHONE_UID']
        except KeyError:
            return None
    jphone_uid = property(get_jphone_uid)

    def parse(self):
        """
        parse the useragent which starts with "SoftBank", "Vodafone", "J-PHONE", or "MOT-".
        """
        # strip crawler infomation such as,
        # J-PHONE/2.0/J-SH03 (compatible; Y!J-SRD/1.0; http://help.yahoo.co.jp/help/jp/search/indexing/indexing-27.html)
        # Vodafone/1.0/V705SH (compatible; Y!J-SRD/1.0; http://help.yahoo.co.jp/help/jp/search/indexing/indexing-27.html)
        # SoftBank/1.0/913SH/SHJ001/SN000123456789000 Browser/NetFront/3.4 Profile/MIDP-2.0 (symphonybot1.froute.jp; +http://search.froute.jp/howto/crawler.html)
        ua = CRAWLER_RE.sub(r'\1', self.useragent)
        ua = ua.strip().split(' ')

        carrier = ua[0]
        if carrier.startswith('SoftBank') or carrier.startswith('Vodafone'):
            self._parse_vodaphone(ua)
        elif carrier.startswith('J-PHONE'):
            self._parse_jphone(ua)
        elif carrier.startswith('MOT-'):
            self._parse_motorola(ua)
        else:
            raise exceptions.NoMatchingError(self)

        try:
            self.msname = self.environ['HTTP_X_JPHONE_MSNAME']
        except KeyError, e:
            self.msname = None

    def make_display(self):
        """
        create a new Display object.
        """
        try:
            width, height = map(int, self.environ['HTTP_X_JPHONE_DISPLAY'].split('*', 1))
        except (KeyError, ValueError, AttributeError):
            width = None
            height = None

        try:
            color_string = self.environ['HTTP_X_JPHONE_COLOR']
            try:
                color = color_string.startswith('C')
            except AttributeError:
                color = False

            try:
                depth = int(color_string[1:])
            except (ValueError, TypeError):
                depth = 0
        except KeyError:
            color = False
            depth = 0

        return Display(width=width, height=height, color=color, depth=depth)

    def _parse_vodaphone(self, ua):
        self.packet_compliant = True

        # Vodafone/1.0/V702NK/NKJ001 Series60/2.6 Nokia6630/2.39.148 Profile/MIDP-2.0 Configuration/CLDC-1.1
        # Vodafone/1.0/V702NK/NKJ001/SN123456789012345 Series60/2.6 Nokia6630/2.39.148 Profile/MIDP-2.0 Configuration/CLDC-1.1
        # Vodafone/1.0/V802SE/SEJ001/SN123456789012345 Browser/SEMC-Browser/4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1
        # Vodafone/1.0/V705SH (compatible; Y!J-SRD/1.0; http://help.yahoo.co.jp/help/jp/search/indexing/indexing-27.html)

        (self.name,
         self.version,
         self.model,
         vendor_version,
         serial) = (ua[0].split('/') + [None, None, None, None])[:5]

        if serial:
            if not serial.startswith('SN'):
                raise exceptions.NoMatchingError(self)
            self.serialnumber = serial[2:]

        if not vendor_version:
            self.vendor = ''
            self.vendor_version = ''
        else:
            self.vendor, self.vendor_version = vendor_version[:-4], vendor_version[-4:]

        self.java_info.update([x.split('/') for x in ua[2:] if x])

    def _parse_jphone(self, ua):
        self._is_3g = False
        if len(ua) > 1:
            # J-PHONE/4.0/J-SH51/SNJSHA3029293 SH/0001aa Profile/MIDP-1.0 Configuration/CLDC-1.0 Ext-Profile/JSCL-1.1.0
            self.packet_compliant = True
            (self.name,
             self.version,
             self.model,
             serial) = (ua[0].split('/') + [None, None, None])[:4]
            if serial:
                if not serial.startswith('SN'):
                    raise exceptions.NoMatchingError(self)
                self.serialnumber = serial[2:]

            try:
                self.vendor, self.vendor_version = ua[1].split('/')
            except ValueError, e:
                raise ValueError(str(ua))
            self.java_info.update([x.split('/') for x in ua[2:]])
        else:
            # J-PHONE/2.0/J-DN02
            self.name, self.version, self.model = ua[0].split('/')
            if self.model:
                matcher = VODAFONE_VENDOR_RE.match(self.model)
                if matcher:
                    self.vendor = matcher.group(1)
                else:
                    matcher = JPHONE_VENDOR_RE.match(self.model)
                    if matcher:
                        self.vendor = matcher.group(1)

    def _parse_motorola(self, ua):
        """
        parse HTTP_USER_AGENT string for the Motorola 3G agent
        """
        self.packet_compliant = True
        self.vendor = 'MOT'

        # MOT-V980/80.2F.2E. MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1
        name, self.vendor_version = ua[0].split('/')
        self.model = name[name.rindex('-')+1:]

        self.java_info.update([x.split('/') for x in ua[2:]])



# -*- coding: utf-8 -*-
import re
import uamobile.exceptions
from uamobile.nonmobile import NonMobileUserAgent as NonMobile
from uamobile.docomo import DoCoMoUserAgent as DoCoMo
from uamobile.ezweb import EZwebUserAgent as EZweb
from uamobile.softbank import SoftBankUserAgent as SoftBank
from uamobile.willcom import WillcomUserAgent as Willcom

__all__ = ['detect', 'NonMobile', 'DoCoMo', 'EZweb', 'SoftBank', 'Willcom']

DOCOMO_RE   = re.compile(r'^DoCoMo/\d\.\d[ /]')
SOFTBANK_RE = re.compile(r'^(?:(?:SoftBank|Vodafone|J-PHONE)/\d\.\d|MOT-)')
EZWEB_RE    = re.compile(r'^(?:KDDI-[A-Z]+\d+[A-Z]? )?UP\.Browser\/')
WILLCOM_RE  = re.compile(r'^Mozilla/3\.0\((?:DDIPOCKET|WILLCOM);|^Mozilla/4\.0 \(compatible; MSIE (?:6\.0|4\.01); Windows CE; SHARP/WS\d+SH; PPC; \d+x\d+\)')

def detect(environ):
    """
    parse HTTP user agent string and detect a mobile device.
    """
    try:
        useragent = environ['HTTP_USER_AGENT']
    except KeyError:
        useragent = ''

    if DOCOMO_RE.match(useragent):
        device = DoCoMo(environ)
    elif EZWEB_RE.match(useragent):
        device = EZweb(environ)
    elif SOFTBANK_RE.match(useragent):
        device = SoftBank(environ)
    elif WILLCOM_RE.match(useragent):
        device = Willcom(environ)
    else:
        device = NonMobile(environ)

    try:
        device.parse()
    except exceptions.NoMatchingError, e:
        raise e
    except ValueError, e:
        raise exceptions.NoMatchingError(device, e)

    return device



# -*- coding: utf-8 -*-
from uamobile import exceptions
from uamobile.base import UserAgent, Display
import re

WILLCOM_RE = re.compile(r'^Mozilla/3\.0\((?:DDIPOCKET|WILLCOM);(.*)\)')
CACHE_RE = re.compile(r'^[Cc](\d+)')
WINDOWS_CE_RE = re.compile(r'^Mozilla/4\.0 \((.*)\)')

class WillcomUserAgent(UserAgent):
    name = 'WILLCOM'
    carrier = 'WILLCOM'
    short_carrier = 'W'

    def __init__(self, *args, **kwds):
        UserAgent.__init__(self, *args, **kwds)
        self.vendor = ''
        self.model_version = ''
        self.browser_version = ''
        self.cache_size = None

        # WILLCOM does not support the serial number of a device
        # except official web sites?
        self.serialnumber = None

    def supports_cookie(self):
        # TODO
        # I'm not sure All WILLCOM phones can handle HTTP cookie.
        return True

    def parse(self):
        if self.useragent.startswith('Mozilla/4.0'):
            return self._parse_windows_ce()

        matcher = WILLCOM_RE.match(self.useragent)
        if not matcher:
            raise exceptions.NoMatchingError(self)

        (self.vendor,
         self.model,
         self.model_version,
         self.browser_version,
         cache) = matcher.group(1).split('/')
        
        matcher = CACHE_RE.match(cache)
        if not matcher:
            raise NoMatchingError(self)

        self.cache_size = int(matcher.group(1))

    def _parse_windows_ce(self):
        #Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; SHARP/WS007SH; PPC; 480x640)
        matcher = WINDOWS_CE_RE.match(self.useragent)
        if not matcher:
            raise exceptions.NoMatchingError(self)

        try:
            xx, ie, xxx, vendor_and_model, xxxx, width_and_height = matcher.group(1).split(';')
            self.vendor, self.model = vendor_and_model.strip().split('/')
        except ValueError, e:
            raise exceptions.NoMatchingError(self)

    def make_display(self):
        """
        create a new Display object.
        """
        return Display()

    def is_willcom(self):
        return True

    def is_airhphone(self):
        return True



# -*- coding: utf-8 -*-

class UserAgentError(Exception):
    pass

class NoMatchingError(UserAgentError):
    def __init__(self, ua=None, error=None):
        self.ua = ua
        self.error = error

    def __repr__(self):
        return '<uamobile.exceptions.NoMatchingError UserAgent:%s>' % self.ua

    def __str__(self):
        return 'Cannot not detect the useragent, "%s", original exception: %s' % (self.ua,
                                                                                  repr(self.error))



# -*- coding: utf-8 -*-
from tests import msg
from uamobile import detect, NonMobile

def test_empty_useragent():
    try:
        ua = detect({})
    except:
        assert False, 'KeyError HTTP_USER_AGENT should be ignored silently'
    else:
        pass

def test_useragent_nonmobile():
    def inner(useragent):
        ua = detect({'HTTP_USER_AGENT':useragent})
        assert isinstance(ua, NonMobile)
        assert ua.carrier == 'NonMobile'
        assert ua.short_carrier == 'N'
        assert ua.is_docomo() == False
        assert ua.is_ezweb() == False
        assert ua.is_softbank() == False
        assert ua.is_vodafone() == False
        assert ua.is_jphone() == False
        assert ua.is_willcom() == False
        assert ua.is_nonmobile()
        assert ua.display is not None
        assert ua.supports_cookie() == True

        assert ua.serialnumber is None

    for ua in DATA:
        yield inner, ua

#########################
# Test data
#########################

DATA = ('Mozilla/2.0 (compatible; Ask Jeeves)',
        'Mozilla/2.0 (compatible; MSIE 3.01; Windows 95)',
        'Mozilla/2.0 (compatible; MSIE 3.02; Windows CE)',
        'Mozilla/2.0 (compatible; MSIE 3.02; Windows CE; 240x320)',
        'Mozilla/2.0 (compatible; MSIE 3.02; Windows CE; 240x320; PPC)',
        'Mozilla/2.0 (compatible; MSIE 3.02; Windows CE; PPC; 240x320)',
        'Mozilla/2.0 (compatible; T-H-U-N-D-E-R-S-T-O-N-E)',
        'Mozilla/3.0 (DreamPassport/3.0)',
        'Mozilla/3.0 (DreamPassport/3.15; SONICTEAM/PSOV2)',
        'Mozilla/3.0 (DreamPassport/3.2)',
        'Mozilla/3.0 (Slurp.so/Goo; slurp@inktomi.com; http://www.inktomi.com/slurp.html)',
        'Mozilla/3.0 (Slurp/si; slurp@inktomi.com; http://www.inktomi.com/slurp.html)',
        'Mozilla/3.0 (Win95; I)',
        'Mozilla/3.0 (Windows 2000; U) Opera 6.05  [ja]',
        'Mozilla/3.0 (aruyo/0.01;http://www.aaacafe.ne.jp/ ;support@aaacafe.ne.jp)',
        'Mozilla/3.0 (compatible)',
        'Mozilla/3.0 (compatible; Indy Library)',
        'Mozilla/3.0 (compatible; NetMind-Minder/4.3.1J)',
        'Mozilla/3.0 (compatible; NetPositive/2.2.1; BeOS)',
        'Mozilla/3.0 (compatible; PerMan Surfer 3.0; Win95)',
        'Mozilla/3.0 (compatible;)',
        'Mozilla/3.01 (compatible;)',
        'Mozilla/3.01 [ja] (Macintosh; I; 68K)',
        'Mozilla/3.01Gold (Macintosh; I; 68K)',
        'Mozilla/3.01Gold (Macintosh; I; 68K; SiteCoach 1.0)',
        'Mozilla/4.0',
        'Mozilla/4.0 (LINKS ARoMATIZED)',
        'Mozilla/4.0 (PDA; SL-A300/1.0,Embedix/Qtopia/1.1.0) NetFront/3.0',
        'Mozilla/4.0 (PDA; Windows CE/0.9.3) NetFront/3.0',
        'Mozilla/4.0 (Windows NT 4.0)',
        'Mozilla/4.0 (compatible',
        'Mozilla/4.0 (compatible; MSIE 4.01; MSN 2.5; Windows 95)',
        'Mozilla/4.0 (compatible; MSIE 4.01; Windows 95)',
        'Mozilla/4.0 (compatible; MSIE 4.01; Windows 98)',
        'Mozilla/4.0 (compatible; MSIE 4.01; Windows NT Windows CE)',
        'Mozilla/4.0 (compatible; MSIE 4.01; Windows NT)',
        'Mozilla/4.0 (compatible; MSIE 4.0; Windows 95)',
        'Mozilla/4.0 (compatible; MSIE 4.5; Mac_PowerPC)',
        'Mozilla/4.0 (compatible; MSIE 5.00; Windows 98)',
        'Mozilla/4.0 (compatible; MSIE 5.01; MSN 2.5; Windows 98)',
        'Mozilla/4.0 (compatible; MSIE 5.01; Windows 95)',
        'Mozilla/4.0 (compatible; MSIE 5.01; Windows 98)',
        'Mozilla/4.0 (compatible; MSIE 5.01; Windows 98; HKBN)',
        'Mozilla/4.0 (compatible; MSIE 5.01; Windows 98; MSIECrawler)',
        'Mozilla/4.0 (compatible; MSIE 5.01; Windows 98; MSOCD; AtHomeJP0109)',
        'Mozilla/4.0 (compatible; MSIE 5.01; Windows 98; Q312461)',
        'Mozilla/4.0 (compatible; MSIE 5.01; Windows 98; YComp 5.0.2.4)',
        'Mozilla/4.0 (compatible; MSIE 5.01; Windows 98; istb 641)',
        'Mozilla/4.0 (compatible; MSIE 5.01; Windows NT 5.0)',
        'Mozilla/4.0 (compatible; MSIE 5.01; Windows NT 5.0) LinkChecker 0.1',
        'Mozilla/4.0 (compatible; MSIE 5.01; Windows NT 5.0) WebWasher 3.2',
        'Mozilla/4.0 (compatible; MSIE 5.01; Windows NT 5.0; .NET CLR 1.0.3705)',
        'Mozilla/4.0 (compatible; MSIE 5.01; Windows NT 5.0; DigExt)',
        'Mozilla/4.0 (compatible; MSIE 5.01; Windows NT 5.0; NetCaptor 7.0.1)',
        'Mozilla/4.0 (compatible; MSIE 5.01; Windows NT 5.0; T312461)',
        'Mozilla/4.0 (compatible; MSIE 5.01; Windows NT 5.0; istb 641)',
        'Mozilla/4.0 (compatible; MSIE 5.01; Windows NT)',
        'Mozilla/4.0 (compatible; MSIE 5.01; Windows NT; Lunascape 0.99c)',
        'Mozilla/4.0 (compatible; MSIE 5.01; Windows NT; Norfolk Southern Corp.)',
        'Mozilla/4.0 (compatible; MSIE 5.01; Windows NT; nk-07102k)',
        'Mozilla/4.0 (compatible; MSIE 5.0; AOL 7.0; Windows 98; DigExt)',
        'Mozilla/4.0 (compatible; MSIE 5.0; Linux 2.2.18-0vl4.2 i686) Opera 6.0  [en]',
        'Mozilla/4.0 (compatible; MSIE 5.0; Mac_PowerPC)',
        'Mozilla/4.0 (compatible; MSIE 5.0; Mac_PowerPC; AtHomeJP191)',
        'Mozilla/4.0 (compatible; MSIE 5.0; Mac_PowerPC;)',
        'Mozilla/4.0 (compatible; MSIE 5.0; Win32)',
        'Mozilla/4.0 (compatible; MSIE 5.0; Windows 2000) Opera 6.0  [ja]',
        'Mozilla/4.0 (compatible; MSIE 5.0; Windows 2000) Opera 6.03  [en]',
        'Mozilla/4.0 (compatible; MSIE 5.0; Windows 2000) Opera 6.03  [ja]',
        'Mozilla/4.0 (compatible; MSIE 5.0; Windows 2000) Opera 6.05  [en]',
        'Mozilla/4.0 (compatible; MSIE 5.0; Windows 2000) Opera 6.05  [ja]',
        'Mozilla/4.0 (compatible; MSIE 5.0; Windows 95)',
        'Mozilla/4.0 (compatible; MSIE 5.0; Windows 95; DigExt)',
        'Mozilla/4.0 (compatible; MSIE 5.0; Windows 98)',
        'Mozilla/4.0 (compatible; MSIE 5.0; Windows 98) Opera 5.12  [es]',
        'Mozilla/4.0 (compatible; MSIE 5.0; Windows 98) Opera 6.03  [ja]',
        'Mozilla/4.0 (compatible; MSIE 5.0; Windows 98) Opera 6.05  [en]',
        'Mozilla/4.0 (compatible; MSIE 5.0; Windows 98) Opera 6.05  [ja]',
        'Mozilla/4.0 (compatible; MSIE 5.0; Windows 98)::ELNSB50::0000211003200258031a018f000000000505000b00000000',
        'Mozilla/4.0 (compatible; MSIE 5.0; Windows 98; DigExt)',
        'Mozilla/4.0 (compatible; MSIE 5.0; Windows 98; DigExt; YComp 5.0.0.0)',
        'Mozilla/4.0 (compatible; MSIE 5.0; Windows 98; Hotbar 3.0)',
        'Mozilla/4.0 (compatible; MSIE 5.0; Windows ME) Opera 6.03  [ja]',
        'Mozilla/4.0 (compatible; MSIE 5.0; Windows ME) Opera 6.05  [ja]',
        'Mozilla/4.0 (compatible; MSIE 5.0; Windows NT 4.0) Opera 6.0  [ja]',
        'Mozilla/4.0 (compatible; MSIE 5.0; Windows NT 4.0) Opera 6.01  [ja]',
        'Mozilla/4.0 (compatible; MSIE 5.0; Windows NT 4.0) Opera 6.03  [ja]',
        'Mozilla/4.0 (compatible; MSIE 5.0; Windows NT 4.0) Opera 6.05  [ja]',
        'Mozilla/4.0 (compatible; MSIE 5.0; Windows NT 5.0)',
        'Mozilla/4.0 (compatible; MSIE 5.0; Windows NT)',
        'Mozilla/4.0 (compatible; MSIE 5.0; Windows NT; DigExt)',
        'Mozilla/4.0 (compatible; MSIE 5.0; Windows NT; DigExt; DTS Agent',
        'Mozilla/4.0 (compatible; MSIE 5.0; Windows XP) Opera 6.01  [de]',
        'Mozilla/4.0 (compatible; MSIE 5.0; Windows XP) Opera 6.03  [ja]',
        'Mozilla/4.0 (compatible; MSIE 5.0; Windows XP) Opera 6.04  [en]',
        'Mozilla/4.0 (compatible; MSIE 5.0; Windows XP) Opera 6.04  [ja]',
        'Mozilla/4.0 (compatible; MSIE 5.0; Windows XP) Opera 6.05  [ja]',
        'Mozilla/4.0 (compatible; MSIE 5.12; Mac_PowerPC)',
        'Mozilla/4.0 (compatible; MSIE 5.14; Mac_PowerPC)',
        'Mozilla/4.0 (compatible; MSIE 5.16; Mac_PowerPC)',
        'Mozilla/4.0 (compatible; MSIE 5.21; Mac_PowerPC)',
        'Mozilla/4.0 (compatible; MSIE 5.22; Mac_PowerPC)',
        'Mozilla/4.0 (compatible; MSIE 5.2; Mac_PowerPC)',
        'Mozilla/4.0 (compatible; MSIE 5.2; Mac_PowerPC) OmniWeb/4.1.1-v424.6',
        'Mozilla/4.0 (compatible; MSIE 5.5; AOL 6.0; Windows 98; Win 9x 4.90)',
        'Mozilla/4.0 (compatible; MSIE 5.5; MSN 2.5; AOL 7.0; Windows 98)',
        'Mozilla/4.0 (compatible; MSIE 5.5; MSN 2.5; Windows 98)',
        'Mozilla/4.0 (compatible; MSIE 5.5; Windows 95)',
        'Mozilla/4.0 (compatible; MSIE 5.5; Windows 95; T312461)',
        'Mozilla/4.0 (compatible; MSIE 5.5; Windows 95; YComp 5.0.0.0)',
        'Mozilla/4.0 (compatible; MSIE 5.5; Windows 95; ie5.5cd_t-zone_0005)',
        'Mozilla/4.0 (compatible; MSIE 5.5; Windows 98)',
        'Mozilla/4.0 (compatible; MSIE 5.5; Windows 98; .NET CLR 1.0.3705)',
        'Mozilla/4.0 (compatible; MSIE 5.5; Windows 98; DigExt)',
        'Mozilla/4.0 (compatible; MSIE 5.5; Windows 98; H010818)',
        'Mozilla/4.0 (compatible; MSIE 5.5; Windows 98; MSIECrawler)',
        'Mozilla/4.0 (compatible; MSIE 5.5; Windows 98; MSN 6.1; MSNbMSFT; MSNmja-jp; MSNc00)',
        'Mozilla/4.0 (compatible; MSIE 5.5; Windows 98; MSOCD; AtHomeJP191)',
        'Mozilla/4.0 (compatible; MSIE 5.5; Windows 98; Q312461)',
        'Mozilla/4.0 (compatible; MSIE 5.5; Windows 98; Q312461; T312461)',
        'Mozilla/4.0 (compatible; MSIE 5.5; Windows 98; T312461)',
        'Mozilla/4.0 (compatible; MSIE 5.5; Windows 98; T312461; .NET CLR 1.0.3705)',
        'Mozilla/4.0 (compatible; MSIE 5.5; Windows 98; T312461; istb 641)',
        'Mozilla/4.0 (compatible; MSIE 5.5; Windows 98; T312461; istb 641; COM+ 1.0.2204)',
        'Mozilla/4.0 (compatible; MSIE 5.5; Windows 98; Win 9x 4.90',
        'Mozilla/4.0 (compatible; MSIE 5.5; Windows 98; Win 9x 4.90)',
        'Mozilla/4.0 (compatible; MSIE 5.5; Windows 98; Win 9x 4.90; .NET CLR 1.0.3705)',
        'Mozilla/4.0 (compatible; MSIE 5.5; Windows 98; Win 9x 4.90; Lunascape 0.98d)',
        'Mozilla/4.0 (compatible; MSIE 5.5; Windows 98; Win 9x 4.90; MSOCD; AtHomeJP0109)',
        'Mozilla/4.0 (compatible; MSIE 5.5; Windows 98; Win 9x 4.90; Q312461)',
        'Mozilla/4.0 (compatible; MSIE 5.5; Windows 98; Win 9x 4.90; Q312461; T312461)',
        'Mozilla/4.0 (compatible; MSIE 5.5; Windows 98; Win 9x 4.90; T312461)',
        'Mozilla/4.0 (compatible; MSIE 5.5; Windows 98; Win 9x 4.90; T312461; .NET CLR 1.0.3705)',
        'Mozilla/4.0 (compatible; MSIE 5.5; Windows 98; Win 9x 4.90; T312461; Lunascape 0.99c)',
        'Mozilla/4.0 (compatible; MSIE 5.5; Windows 98; Win 9x 4.90; Unithink)',
        'Mozilla/4.0 (compatible; MSIE 5.5; Windows 98; Win 9x 4.90; telus.net_v5.0.1; Hotbar 4.0)',
        'Mozilla/4.0 (compatible; MSIE 5.5; Windows 98; telus.net_v5.0.1)',
        'Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 4.0)',
        'Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 4.0; H010818)',
        'Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 4.0; H010818; CPT-IE401SP1; T312461)',
        'Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 4.0; SenseWave 1.0)',
        'Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 4.0; Suncorp Metway Ltd)',
        'Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 4.0; T312461)',
        'Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 4.0; T312461; .NET CLR 1.0.3705)',
        'Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 4.0; T312461; Lunascape 0.95a)',
        'Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 4.0; Yahoo! JAPAN Version Windows 95/NT CD-ROM Edition 1.0.)',
        'Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 5.0)',
        'Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 5.0; (R1 1.1))',
        'Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 5.0; (R1 1.1); (R1 1.3))',
        'Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 5.0; (R1 1.3))',
        'Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 5.0; .NET CLR 1.0.3705)',
        'Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 5.0; AIRF)',
        'Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 5.0; DigExt)',
        'Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 5.0; FORJEIS55SP1)',
        'Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 5.0; H010818)',
        'Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 5.0; Hotbar 3.0; istb 641)',
        'Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 5.0; Hotbar 4.1.7.0)',
        'Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 5.0; Lunascape 0.98c)',
        'Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 5.0; N_o_k_i_a)',
        'Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 5.0; Q312461)',
        'Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 5.0; Q312461; .NET CLR 1.0.3705)',
        'Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 5.0; T312461)',
        'Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 5.0; T312461; .NET CLR 1.0.3705)',
        'Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 5.0; T312461; Hewle'
        )



# -*- coding: utf-8 -*-

class UserAgent(object):
    """
    Base class representing HTTTP user agent.
    """

    def __init__(self, environ):
        try:
            self.useragent = environ['HTTP_USER_AGENT']
        except KeyError, e:
            self.useragent = ''
        self.environ = environ
        self.model = ''
        self.version = ''
        self._display = None

    def __repr__(self):
        return '<%s "%s">' % (self.__class__.__name__, self.useragent)

    def __str__(self):
        return self.useragent

    def get_display(self):
        """
        returns Display object.
        """
        if self._display is None:
            self._display = self.make_display()
        return self._display
    display = property(get_display)

    def make_display(self):
        raise NotImplementedError

    def is_docomo(self):
        """
        returns True if the agent is DoCoMo.
        """
        return False

    def is_ezweb(self):
        """
        returns True if the agent is EZweb.
        """
        return False

    def is_tuka(self):
        """
        returns True if the agent is TU-Ka.
        """
        return False

    def is_softbank(self):
        """
        returns True if the agent is Softbank.
        """
        return False

    def is_vodafone(self):
        """
        returns True if the agent is Vodafone (now SotBank).
        """
        return False

    def is_jphone(self):
        """
        returns True if the agent is J-PHONE (now softbank).
        """
        return False

    def is_willcom(self):
        """
        returns True if the agent is Willcom.
        """
        return False

    def is_airhphone(self):
        """
        returns True if the agent is AirH'PHONE.
        """
        return False

    def is_wap1(self):
        return False

    def is_wap2(self):
        return False

    def is_nonmobile(self):
        return False

    def supports_cookie(self):
        """
        returns True if the agent supports HTTP cookie.
        """
        raise NotImplementedError

class Display(object):
    """
    Display information for mobile devices.
    """

    def __init__(self, width=None, height=None, depth=None, color=None,
                 width_bytes=None, height_bytes=None):
        self.width = width or 0
        self.height = height or 0
        self.depth = depth or 0
        self.color = color or 0
        self.width_bytes = width_bytes
        self.height_bytes = height_bytes

    def is_qvga(self):
        return self.width >= 240

    def is_vga(self):
        return self.width >= 480



# -*- coding: utf-8 -*-
from tests import msg
from uamobile import detect, exceptions, SoftBank

def test_display():
    env = {'HTTP_USER_AGENT': 'Vodafone/1.0/V904SH/SHJ003/SN000000000000000 Browser/VF-NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1',
           'HTTP_X_JPHONE_MSNAME': 'V904SH',
           'HTTP_X_JPHONE_COLOR': 'C262144',
           'HTTP_X_SSL': 'off',
           'HTTP_X_JPHONE_REGION': '44020',
           'HTTP_X_JPHONE_DISPLAY': '480*640',
           'HTTP_X_JPHONE_UID': 'xxxxxxxxxxxxxxxx',
           'HTTP_X_JPHONE_SMAF': '128/pcm/grf/rs',
           'HTTP_X_WAP_PROFILE': 'http://www.sharp-mobile.com/UAProf/V904SH_SHJ001_3g.xml'
           }
    ua = detect(env)
    assert ua.display.width == 480
    assert ua.display.height == 640
    assert ua.display.color == True
    assert ua.display.depth == 262144, "%s expected, result %r" % (262144, ua.display.color)
    assert ua.display.is_qvga() == True
    assert ua.display.is_vga() == True

def test_display_error():
    base_env = {'HTTP_USER_AGENT': 'Vodafone/1.0/V904SH/SHJ003/SN000000000000000 Browser/VF-NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1',
                'HTTP_X_JPHONE_COLOR': 'C262144',
                'HTTP_X_JPHONE_DISPLAY': '480*640'
           }

    def func(ua, width, height, color, depth):
        assert ua.display.width == width
        assert ua.display.height == height
        assert ua.display.color == color
        assert ua.display.depth == depth

    # Invalid COLOR
    env = dict(base_env)
    del env['HTTP_X_JPHONE_COLOR']
    ua = detect(env)
    yield (func, ua, 480, 640, False, 0)

    env = dict(base_env)
    env['HTTP_X_JPHONE_COLOR'] = 'spam'
    ua = detect(env)
    yield (func, ua, 480, 640, False, 0)

    env = dict(base_env)
    env['HTTP_X_JPHONE_COLOR'] = ''
    ua = detect(env)
    yield (func, ua, 480, 640, False, 0)

    env = dict(base_env)
    env['HTTP_X_JPHONE_COLOR'] = 1
    ua = detect(env)
    yield (func, ua, 480, 640, False, 0)

    # Invalid DISPLAY
    env = dict(base_env)
    del env['HTTP_X_JPHONE_DISPLAY']
    ua = detect(env)
    yield (func, ua, 0, 0, True, 262144)

    env = dict(base_env)
    env['HTTP_X_JPHONE_DISPLAY'] = '480'
    ua = detect(env)
    yield (func, ua, 0, 0, True, 262144)

    env = dict(base_env)
    env['HTTP_X_JPHONE_DISPLAY'] = '480*spam'
    ua = detect(env)
    yield (func, ua, 0, 0, True, 262144)

    # Invalid type
    env = dict(base_env)
    env['HTTP_X_JPHONE_DISPLAY'] = 1
    ua = detect(env)
    yield (func, ua, 0, 0, True, 262144)

def test_jphone_uid():
    useragent = 'Vodafone/1.0/V904SH/SHJ003/SN000000000000000 Browser/VF-NetFront/3.3 Profile/MIDP-2.0 Configuration/CLDC-1.1'
    ua = detect({'HTTP_USER_AGENT': useragent,
                 'HTTP_X_JPHONE_UID': 'xxxxxxxxxxxxxxxx' })
    assert ua.jphone_uid == 'xxxxxxxxxxxxxxxx'

    ua = detect({'HTTP_USER_AGENT': useragent })
    assert ua.jphone_uid is None

def test_useragent_softbank():
    def inner(useragent, version, model, packet_compliant,
              serial_number=None, vendor=None, vendor_version=None, java_infos=None):
        ua = detect({'HTTP_USER_AGENT': useragent})

        assert isinstance(ua, SoftBank)
        assert ua.carrier == 'SoftBank'
        assert ua.short_carrier == 'S'

        assert ua.is_docomo() == False
        assert ua.is_ezweb() == False
        assert ua.is_softbank()
        assert ua.is_vodafone()
        assert ua.is_jphone()
        assert ua.is_willcom() == False
        assert ua.is_nonmobile() == False
        assert ua.display is not None

        if serial_number:
            assert ua.serialnumber == serial_number, msg(ua, ua.serialnumber, serial_number)
            assert ua.version == version
            assert ua.model == model, msg(ua, ua.model, model)
            assert ua.packet_compliant == packet_compliant

            assert ua.vendor == vendor, msg(ua, ua.vendor, vendor)
            assert ua.vendor_version == vendor_version, msg(ua, ua.vendor_version, vendor_version)

    for args in DATA:
        yield ([inner] + list(args))

def test_jphone_2_0():
    ua = detect({'HTTP_USER_AGENT':'J-PHONE/2.0/J-DN02'})
    assert isinstance(ua.version, basestring)
    assert not ua.is_3g(), 'Invalid generation'
    assert ua.is_type_c(), 'Invalid type %s' % ua.version
    assert not ua.is_type_p(), 'Invalid type %s'  % ua.version
    assert not ua.is_type_w(), 'Invalid type' % ua.version
    assert ua.supports_cookie() == False
    assert ua.vendor == 'DN', ua.vendor

def test_jphone_3_0():
    ua = detect({'HTTP_USER_AGENT':'J-PHONE/3.0/J-PE03_a'})
    assert not ua.is_3g(), 'Invalid generation'
    assert ua.is_type_c(), 'Invalid type %s' % ua.version
    assert not ua.is_type_p(), 'Invalid type %s'  % ua.version
    assert not ua.is_type_w(), 'Invalid type' % ua.version
    assert ua.supports_cookie() == False
    assert ua.vendor == 'PE', ua.vendor

def test_jphone_4_0():
    ua = detect({'HTTP_USER_AGENT':'J-PHONE/4.0/J-SH51/SNJSHA3029293 SH/0001aa Profile/MIDP-1.0 Configuration/CLDC-1.0 Ext-Profile/JSCL-1.1.0'})
    assert not ua.is_type_c(), 'Invalid type %s' % ua.version
    assert ua.is_type_p(), 'Invalid type %s'  % ua.version
    assert not ua.is_type_w(), 'Invalid type' % ua.version
    assert ua.supports_cookie() == False
    assert ua.vendor == 'SH'

def test_jphone_5_0():
    ua = detect({'HTTP_USER_AGENT':'J-PHONE/5.0/V801SA'})
    assert not ua.is_type_c(), 'Invalid type %s' % ua.version
    assert not ua.is_type_p(), 'Invalid type %s'  % ua.version
    assert ua.is_type_w(), 'Invalid type' % ua.version
    assert ua.supports_cookie() == True
    assert ua.vendor == 'SA'

def test_vodafone_1_0():
    ua = detect({'HTTP_USER_AGENT':'Vodafone/1.0/V702NK/NKJ001 Series60/2.6 Nokia6630/2.39.148 Profile/MIDP-2.0 Configuration/CLDC-1.1'})
    assert ua.is_3g(), 'Invalid generation'
    assert not ua.is_type_c(), 'Invalid type %s' % ua.version
    assert not ua.is_type_p(), 'Invalid type %s'  % ua.version
    assert not ua.is_type_w(), 'Invalid type' % ua.version
    assert ua.supports_cookie() == True

def test_yahoo_crawler():
    def func(useragent, is_3g, model, cookie):
        ua = detect({'HTTP_USER_AGENT': useragent})
        assert ua.is_softbank()
        assert ua.is_3g() == is_3g, 'Invalid generation'
        assert not ua.is_type_p(), 'Invalid type %s'  % ua.version
        assert not ua.is_type_w(), 'Invalid type' % ua.version
        assert ua.model == model
        assert ua.supports_cookie() == cookie

    for u, model, is_3g, cookie in (
        ('J-PHONE/2.0/J-SH03 (compatible; Y!J-SRD/1.0; http://help.yahoo.co.jp/help/jp/search/indexing/indexing-27.html)', False, 'J-SH03', False),
        ('Vodafone/1.0/V705SH (compatible; Y!J-SRD/1.0; http://help.yahoo.co.jp/help/jp/search/indexing/indexing-27.html)', True, 'V705SH', True),
        ):
        yield func, u, model, is_3g, cookie

def test_crawler():
    def func(useragent):
        ua = detect({'HTTP_USER_AGENT':useragent})
        assert ua.is_softbank()
    for useragent in ('Vodafone/1.0/V802SH/SHJ002 Browser/UP.Browser/7.0.2.1 (compatible; ichiro/mobile goo; +http://help.goo.ne.jp/door/crawler.html)',
                      'J-PHONE/3.0/J-SH10 (compatible; LD_mobile_bot; +http://helpguide.livedoor.com/help/search/qa/grp627)',
                      'SoftBank/1.0/913SH/SHJ001/SN000123456789000 Browser/NetFront/3.4 Profile/MIDP-2.0 (symphonybot1.froute.jp; +http://search.froute.jp/howto/crawler.html)',
                      ):
        yield func, useragent

def test_error_agents():
    def tester(useragent):
        try:
            ua = detect({'HTTP_USER_AGENT':useragent})
        except exceptions.NoMatchingError:
            pass
        else:
            assert False, 'NoMatchingError expected'
    for datum in ERRORS:
        yield tester, datum

#########################
# Test data
#########################

DATA = (
# ua, version, model, packet_compliant, serial_number, vendor, vendor_version, java_infos
    ('J-PHONE/2.0/J-DN02', '2.0', 'J-DN02', False),
    ('J-PHONE/3.0/J-PE03_a', '3.0', 'J-PE03_a', False),
    ('J-PHONE/4.0/J-SH51/SNJSHA3029293 SH/0001aa Profile/MIDP-1.0 Configuration/CLDC-1.0 Ext-Profile/JSCL-1.1.0', '4.0', 'J-SH51', True, 'JSHA3029293', 'SH', '0001aa', { 'Profile':'MIDP-1.0', 'Configuration':'CLDC-1.0', 'Ext-Profile':'JSCL-1.1.0' }),
    ('J-PHONE/4.0/J-SH51/SNXXXXXXXXX SH/0001a Profile/MIDP-1.0 Configuration/CLDC-1.0 Ext-Profile/JSCL-1.1.0', '4.0', 'J-SH51', True, 'XXXXXXXXX', 'SH', '0001a', {'Profile' : 'MIDP-1.0', 'Configuration' : 'CLDC-1.0', 'Ext-Profile' : 'JSCL-1.1.0'}),
    ('J-PHONE/5.0/V801SA SA/0001JP Profile/MIDP-1.0 Configuration/CLDC-1.0 Ext-Profile/JSCL-1.1.0', '5.0', 'V801SA', True),
    ('Vodafone/1.0/V702NK/NKJ001 Series60/2.6 Nokia6630/2.39.148 Profile/MIDP-2.0 Configuration/CLDC-1.1', '1.0', 'V702NK', True, None, 'NK', 'NKJ001', {'Profile' : 'MIDP-1.0', 'Configuration' : 'CLDC-1.0'}),

    # TODO check whether this serial number is valid.
    ('Vodafone/1.0/V702NK/NKJ001/SN123456789012345 Series60/2.6 Nokia6630/2.39.148 Profile/MIDP-2.0 Configuration/CLDC-1.1', '1.0', 'V702NK', True, '123456789012345', 'NK', 'J001', {'Profile' : 'MIDP-1.0', 'Configuration' : 'CLDC-1.0'}),

    # TODO cehck whether this serial number is valid
    ('Vodafone/1.0/V802SE/SEJ001/SN123456789012345 Browser/SEMC-Browser/4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1', '1.0', 'V802SE', True, '123456789012345', 'SE', 'J001', {'Profile' : 'MIDP-2.0', 'Configuration' : 'CLDC-1.1'}),
    ('Vodafone/1.0/V902SH/SHJ001 Browser/UP.Browser/7.0.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1 Ext-J-Profile/JSCL-1.2.2 Ext-V-Profile/VSCL-2.0.0', '1.0', 'V902SH', True, None, 'SH', 'J001', {'Profile' : 'MIDP-2.0', 'Configuration' : 'CLDC-1.1'}),
    ('Vodafone/1.0/V802N/NJ001 Browser/UP.Browser/7.0.2.1.258 Profile/MIDP-2.0 Configuration/CLDC-1.1 Ext-J-Profile/JSCL-1.2.2 Ext-V-Profile/VSCL-2.0.0', '1.0', 'V802N', True, None, 'N', 'J001', {'Profile' : 'MIDP-2.0', 'Configuration' : 'CLDC-1.1'}),
    ('MOT-V980/80.2F.2E. MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1', '', 'V980', True, None, 'MOT', '80.2F.2E.', {'Profile' : 'MIDP-2.0', 'Configuration' : 'CLDC-1.1'}),
    ('J-PHONE/3.0/V301SH', '3.0', 'V301SH', False),
    ('J-PHONE/3.0/V301T', '3.0', 'V301T', False),
    ('SoftBank/1.0/705P/PJP10 Browser/Teleca-Browser/3.1 Profile/MIDP-2.0 Configuration/CLDC-1.1', '1.0', '705P', True, None, 'P', 'JP10', {'Profile' : 'MIDP-2.0', 'Configuration' : 'CLDC-1.1'}),
    ('SoftBank/1.0/DM001SH/SHJ001/SN123456789012345 Browser/NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1', '1.0', 'DM001SH', True, '123456789012345', 'SH', 'J001', {'Profile': 'MIDP-2.0', 'Configuration': 'CLDC-1.1'}),
)

ERRORS = ('J-PHONE/4.0/J-SH51_a/ZNJSHA5081372 SH/0001aa Profile/MIDP-1.0 Configuration/CLDC-1.0 Ext-Profile/JSCL-1.1.0',
          #'Vodafone/1.0/V702NK',
          'Vodafone/1.0/V702NK/NKJ001/123456789012345 Series60/2.6 Nokia6630/2.39.148 Profile/MIDP-2.0 Configuration/CLDC-1.1',
          )



#!/usr/bin/env python
import re
import urllib
import pprint
from lxml import etree
from StringIO import StringIO

URL = 'http://www.nttdocomo.co.jp/service/imode/make/content/spec/screen_area/index.html'

MODEL_RE   = re.compile(r'^[A-Z]{1,2}\d+')
DISPLAY_RE = re.compile(r'[^\d]*(\d+)[^\d]+(\d+).*$', re.S)
COLOR_RE   = re.compile(r'[^\d]+(\d+)')

def build_displaymap():
    content = unicode(urllib.urlopen(URL).read(), 'cp932')

    for number, zenkaku in enumerate(u'\uff10\uff11\uff12\uff13\uff14\uff15\uff16\uff17\uff18\uff19'):
        content = content.replace(zenkaku, str(number))

    tree = etree.parse(StringIO(content), etree.HTMLParser())

    displaymap = {}
    for node in tree.xpath('//tr[@class="acenter"]'):
        td = node.xpath('td')

        model = MODEL_RE.match(td[1][0].text) and td[1][0].text or td[0][0].text
        model = model.replace(u'\u03bc', u'myu').replace(u'\uff08', u' ').split()[0]

        display_text = u''.join(td[-3][0].itertext())
        width, height = DISPLAY_RE.match(display_text).groups()

        color_text = u''.join(td[-1][0].itertext())
        color = color_text.startswith(u'\u30ab\u30e9\u30fc')
        depth = COLOR_RE.match(color_text).group(1)

        displaymap[model.encode('ascii')] = dict(width=int(width),
                                                 height=int(height),
                                                 color=int(color),
                                                 depth=int(depth),
                                                 )

    return displaymap

if __name__ == '__main__':
    print "DISPLAYMAP_DOCOMO = %s" % pprint.pformat(build_displaymap())



# -*- coding: utf-8 -*-
from uamobile import exceptions
from uamobile.base import UserAgent, Display

import re

UP_BROWSER_COMMENT_RE = re.compile('^\((.*)\)$')

class EZwebUserAgent(UserAgent):
    carrier = 'EZweb'
    short_carrier = 'E'

    def __init__(self, *args, **kwds):
        UserAgent.__init__(self, *args, **kwds)
        self._comment = None
        self.xhtml_compliant = False
        self.device_id = ''
        self.server = ''

    def supports_cookie(self):
        return True

    def parse(self):
        if self.useragent.startswith('KDDI-'):
            # KDDI-TS21 UP.Browser/6.0.2.276 (GUI) MMP/1.1
            # KDDI-TS3A UP.Browser/6.2.0.11.2.1 (GUI) MMP/2.0, Mozilla/4.08 (MobilePhone; NMCS/3.3) NetFront/3.3
            useragent = self.useragent[5:]
            self.xhtml_compliant = True
            self.device_id, browser, opt, self.server = useragent.split(' ', 4)[:4]
            if self.server.endswith(','):
                self.server = self.server[:-1]
            self._name, version = browser.split('/')
            self.version = '%s %s' % (version, opt)
        else:
            # UP.Browser/3.01-HI01 UP.Link/3.4.5.2
            browser, self.server, comment = (self.useragent.split(' ', 2) + [None, None])[:3]
            self._name, software = browser.split('/', 1)
            self.version, self.device_id = software.split('-', 1)
            if comment:
                self._comment = UP_BROWSER_COMMENT_RE.sub(r'\1', comment)

        self.model = self.device_id

    def make_display(self):
        """
        create a Display object.
        """
        env = self.environ
        try:
            width, height = map(int, env['HTTP_X_UP_DEVCAP_SCREENPIXELS'].split(',', 1))
        except (KeyError, ValueError), e:
            width = None
            height = None

        try:
            color = env['HTTP_X_UP_DEVCAP_ISCOLOR'] == '1'
        except KeyError:
            color = False

        try:
            sd = env['HTTP_X_UP_DEVCAP_SCREENDEPTH'].split(',', 1)
            depth = sd[0] and (2 ** int(sd[0])) or 0
        except (KeyError, ValueError):
            depth = None

        return Display(width=width, height=height, color=color, depth=depth)

    def is_ezweb(self):
        return True

    def get_serialnumber(self):
        """
        return the EZweb subscriber ID.
        if no subscriber ID is available, returns None.
        """
        try:
            return self.environ['HTTP_X_UP_SUBNO']
        except KeyError:
            return None
    serialnumber = property(get_serialnumber)

    def get_comment(self):
        """
        returns comment link 'Google WAP Proxy/1.0'.
        if no comment, then returns None.
        """
        return self._comment
    comment = property(get_comment)

    def is_xhtml_compliant(self):
        """
        returns whether it's XHTML compliant or not.
        """
        return self.xhtml_compliant

    def is_win(self):
        """
        returns whether the agent is CDMA 1X WIN or not.
        """
        return self._device_id[2:1] == '3'

    def is_wap1(self):
        return not self.is_wap2()

    def is_wap2(self):
        return self.xhtml_compliant
